      PROGRAM CALL_FIT 
      IMPLICIT  NONE
      !--------------------------------------------------------------------
      ! O. A. RONDON 10/2005
      !
      ! Make  a Least-Squares fit to a non-linear function
      ! with a linearization of the fitting function.
      ! Page 237 Data Reduction and error analysis for the physical
      ! sciences, Philip R. Bevington, McGraw-Hill, 1969. 
      ! 
      ! Resonance and DIS functional form follow Stein, PRD 12 (1975) 1884
      ! 03/07 KJS: Modified DIS, Resonance functions.
      !            Modified definition of Chi2 in Curfit.
      !            Added calculation of derivs following Mark's code.
      !            Removed Squre root from error matrix
      !       
      ! IMODE : Method of weighting Least Squares Fit
      !        +1 Weight(I) = 1./SIGMAY(I)**2
      !         0 Weight(I) = 1.
      !        -1 Weight(I) = 1./Y(I)
      !--------------------------------------------------------------------

      INCLUDE    'declare.h'

      INTEGER    IDIS,NDIS,NRES,IMODE
      REAL*8      ERR_MAT1(MAX,MAX)
      REAL*8      ERR_MAT2(MAX,MAX)
      REAL*8    PARS1_INIT(MAX)
      REAL*8    PARS2_INIT(MAX)
      REAL*8         PARS1(MAX)        
      REAL*8         PARS2(MAX)       
      REAL*8        EPARS1(MAX)            
      REAL*8        EPARS2(MAX)
      REAL*8    UNUSED1,UNUSED2

      REAL*8    A1A2FIT,FUNCTN,A1_OR,A2_OR,CHI1,CHI2
      REAL*8    D1,R21,R31,R41,DIS1
      REAL*8    D2,R22,R32,R42,DIS2
      REAL*8    AR(NBW,NV), WR(NBW,NV), GR(NBW,NV), BB(5,NV), AA(NV)

      REAL*8    XW_V(NX),Q2_V(NX),A1(NX),EA1(NX),A2(NX),EA2(NX),YFIT(NX)
      REAL*8 DEL,A1_LO,A1_HI,DERIV1(NX,MAX),DERIV2(NX,MAX),A2_LO,A2_HI
      REAL*8 C1(NX),C2(NX), derr,calc_deriv,DWLIM

      INTEGER   J,NP,IA1A2,NPAR,I,II,JJ,K

      CHARACTER DATAFILE*80, PARAMFILE*80,OUTFILE1*80,OUTFILE2*80,CDIS*2
      LOGICAL   ERR1, ERR2

      CHARACTER*13 PARNAMES(17)
      DATA PARNAMES/'Peak Height 1','Peak Height 2','Peak Height 3','Peak Height 4',
     &              'Peak Center 1','Peak Center 2','Peak Center 3','Peak Center 4',
     &              'Peak Width  1','Peak Width  2','Peak Width  3','Peak Width  4',
     &              'Dis 1        ','Dis 2        ','Dis 3        ','Dis 4        ',
     &              'Dis 5        '                                                /



      COMMON/FITPARS/AR,WR,GR,BB,AA,PARS1,PARS2,PARS1_INIT,PARS2_INIT
      COMMON/DISFLAG/IDIS,NDIS



      !------------------------------------------------------------------------     
      DEL     = 0.05
      DWLIM   = 0.150           ! GeV.  Restricts A2 fit to have Delta width atleast this big.
      IMODE   = 1               ! Error Weighting method.
      IDIS    = 05              ! Which DIS functional form to use in GET_DIS

      CALL GET_NDIS()           ! Number of DIS parameters
      NRES    = NBW*3           ! Number of Resonance parameters
      NPAR    = NRES+NDIS       ! Total Number of fit parameters

      CALL GET_NDIS()           ! Number of DIS parameters for a given IDIS.
      CALL CONVERT(IDIS,CDIS)

      OPEN(1, FILE= 'input/dat/rss_proton_b2.dat',             STATUS='old')
      OPEN(2, FILE= 'input/par/fitpars.'//CDIS,                STATUS='old')
      OPEN(92,FILE='output/error_matrix_idis'//CDIS//'_b2.out'             )
      OPEN(96,FILE='output/fit_'//CDIS//'.out'                             )


      ! Read in the data to fit.
      ! ---------------------------------------------
      CALL GET_DATA(NP,XW_V,Q2_V, A1,EA1, A2,EA2)    
      CLOSE(1)


      ! Read in the initial guess for fit parameters.
      ! ---------------------------------------------
      DO J=1,NRES
        READ(2,*) PARS1(J),PARS2(J)
      ENDDO
      DO J=1,NDIS,+1
        READ(2,*) PARS1(NRES+J),PARS2(NRES+J)
      ENDDO
      CLOSE(2)


      ! Fit A1
      !-----------------------------------------------
      IA1A2=1
      CALL FIT(XW_V,Q2_V, A1,EA1,NP,NPAR,IMODE,PARS1,EPARS1,YFIT,CHI1,ERR_MAT1,IA1A2,DWLIM)


    
      WRITE(92,'(A,I10 )') '# IDIS      ',IDIS
      WRITE(92,'(A,F7.2)') '# A1 fit Chi2/N : ', CHI1
      WRITE(92,'(A)')      '#'
      WRITE(92,'(A)')      '# A1 Parameters with errors '

      DO J=1,NPAR
        WRITE(92,'(2F10.3,2A)') PARS1(J), EPARS1(J),'       ! ',PARNAMES(J)
      ENDDO

      WRITE(92,'(A)') '#'
      WRITE(92,'(A)') '# A1 Error Matrix (square root)'
      DO JJ = 1,NPAR     
        WRITE(92,'(18E15.4)') (SQRT(ABS(ERR_MAT1(JJ,II))),II=1,JJ)         ! KJS
      ENDDO



      ! Fit A2
      !-----------------------------------------------
      IA1A2=2
      CALL FIT(XW_V,Q2_V, A2,EA2,NP,NPAR,IMODE,PARS2,EPARS2,YFIT,CHI2,ERR_MAT2,IA1A2,DWLIM)


      WRITE(92,'(A,F7.2)') '# A2 fit Chi2/N : ', CHI2
      WRITE(92,'(A)') '#'
      WRITE(92,'(A)') '# A2 Parameters with errors '

      DO J=1,NPAR
        WRITE(92,'(2F10.3,2A)') PARS2(J), EPARS2(J),'       ! ',PARNAMES(J)
      ENDDO

      WRITE(92,'(A)') '#'
      WRITE(92,'(A)') '# A2 Error Matrix (square root)'
      DO JJ = 1,NPAR
        WRITE(92,'(18E15.4)') (SQRT(ABS(ERR_MAT2(JJ,II))),II=1,JJ)
      ENDDO


      WRITE(6,'(I5,2F10.2)') IDIS, CHI1,CHI2


      ! Calculate derivatives following method in Mark's code.
      ! ------------------------------------------------------
      DO J=1,NP
        DO I=1,NPAR
          PARS1(I)   = PARS1(I) * (1.-DEL)
          A1_LO      = FUNCTN(XW_V,Q2_V,J,PARS1,NPAR,1)

          PARS1(I)   = PARS1(I) * (1.+ DEL)/(1.-DEL)
          A1_HI      = FUNCTN(XW_V,Q2_V,J,PARS1,NPAR,1)

          PARS1(I)   = PARS1(I) / (1.+ DEL)             ! Reset pars to original value

          DERIV1(J,I) = (A1_HI-A1_LO)/(PARS1(I)*2.*DEL)

        ENDDO
      ENDDO

      ! Use covariant matrix to find fit error, including correlations between params.
      DO J=1,NP
         C1(J) = 0.0
         DO I=1,NPAR
           DO K=1,NPAR
             C1(J) = C1(J) + DERIV1(J,I)*ERR_MAT1(I,K)*DERIV1(J,K)
           ENDDO
         ENDDO
         C1(J) = SQRT(C1(J))
      ENDDO


      ! A2
      DO J=1,NP
        DO I=1,NPAR
          PARS2(I)   = PARS2(I) * (1.-DEL)
          A2_LO      = FUNCTN(XW_V,Q2_V,J,PARS2,NPAR,2)

          PARS2(I)   = PARS2(I) * (1.+ DEL)/(1.-DEL)
          A2_HI      = FUNCTN(XW_V,Q2_V,J,PARS2,NPAR,2)

          PARS2(I)   = PARS2(I) / (1.+ DEL)             ! Reset pars to original value

          DERIV2(J,I) = (A2_HI-A2_LO)/(PARS2(I)*2.*DEL)

        ENDDO
      ENDDO

      DO J=1,NP
         C2(J) = 0.0
         DO I=1,NPAR
           DO K=1,NPAR
             C2(J) = C2(J) + DERIV2(J,I)*ERR_MAT2(I,K)*DERIV2(J,K)
           ENDDO
         ENDDO
         C2(J) = SQRT(C2(J))
      ENDDO




      !---------------------------------------------------------------------







      
      DO J=1,NBW ! Res
        AR(J,1) = PARS1(J)
        AR(J,2) = PARS2(J)
        WR(J,1) = PARS1(J+4)
        WR(J,2) = PARS2(J+4)
        GR(J,1) = PARS1(J+8)
        GR(J,2) = PARS2(J+8)
      ENDDO

      DO J=1,NDIS ! DIS
        BB(J,1) = PARS1(J+12)
        BB(J,2) = PARS2(J+12) 
      ENDDO

      DO J=1,NP
        A1_OR = A1A2FIT(1,XW_V(J),Q2_V(J),D1,R21,R31,R41,DIS1,ERR1)
        A2_OR = A1A2FIT(2,XW_V(J),Q2_V(J),D2,R22,R32,R42,DIS2,ERR2)
        WRITE(96,'(30E15.3)') XW_V(J),
     &        A1(J),EA1(J),DIS1,A1_OR,A1_OR+C1(J),A1_OR-C1(J)
     &       ,A2(J),EA2(J),DIS2,A2_OR,A2_OR+C2(J),A2_OR-C2(J)   

      ENDDO
 
    
      RETURN
      END   



!======================================================================
      
      REAL*8 FUNCTION A1A2FIT(K,W,QSQ,D,R2,R3,R4,DIS,ERR)
      IMPLICIT NONE

      !-----------------------------------------------------------------
      ! FUNCTION TO OUTPUT A1(W) AND A2(W) FITS TO RSS DATA
      ! O. A. RONDON 10/2005
      ! INPUTS: W [GEV], Q**2 [GEV**2]
      ! K = WHICH ASYMMETRY: 1 = A1, 2 = A2
      ! LOGICAL ERR CAN BE USED TO SKIP W VALUES BELOW PI0 THRESHOLD
      ! B-W PARAMETERS: AR = AMPLITUDE
      !                 WR = CENTROID
      !                 GR = WIDTH
      !                 DIS BACKGROUND = X**A*(B0+(B1+B2*X+B3*X**2)*X)
      !-----------------------------------------------------------------

      INCLUDE 'declare.h'

      REAL*8      M,MPI
      PARAMETER ( M   = 0.93828   )
      PARAMETER ( MPI = 0.134     )
     
      REAL*8 PARS1(MAX),PARS1_INIT(MAX)
      REAL*8 PARS2(MAX),PARS2_INIT(MAX)

      REAL*8  AR(NBW,NV), WR(NBW,NV), GR(NBW,NV), B(5,NV), A(NV)
      REAL*8  D,R2,R3,R4,DIS
      REAL*8  W,XB,QSQ

      INTEGER K
      LOGICAL ERR

      REAL*8 GET_DIS,GET_BW

      COMMON/FITPARS/AR,WR,GR,B,A,PARS1,PARS2,PARS1_INIT,PARS2_INIT

      !-------------------------------------------------------
      
      ERR = .FALSE.
      IF(W.LT.(M+MPI)) THEN
        ERR = .TRUE.
        RETURN
      END IF

      
      XB      = ((W**2-M**2)/QSQ + 1.0)**(-1)

      D       = GET_BW(W,QSQ,AR(1,K),WR(1,K),GR(1,K),1)
      R2      = GET_BW(W,QSQ,AR(2,K),WR(2,K),GR(2,K),2)
      R3      = GET_BW(W,QSQ,AR(3,K),WR(3,K),GR(3,K),3)
      R4      = GET_BW(W,QSQ,AR(4,K),WR(4,K),GR(4,K),4)
      DIS     = GET_DIS(K,XB,QSQ,B(1,K),B(2,K),B(3,K),B(4,K),B(5,K))

      A1A2FIT = (D + R2 + R3 + R4) + DIS
      RETURN
      END
 




      REAL*8 FUNCTION FUNCTN(X1,X2,INDEX,A,NPARS,IA1A2)
      IMPLICIT NONE
C
C     O. A. RONDON 10/2005
C
      INCLUDE 'declare.h'
      REAL*8 M,MPI
      PARAMETER ( M   = 0.93828 )
      PARAMETER ( MPI = 0.134   )

      INTEGER NPARS,INDEX,IA1A2,J
      REAL*8 X1(1),X2(1)
      REAL*8 A(NPARS)
      REAL*8 AR(NBW), WR(NBW), GR(NBW), B(5),ALPHA
      REAL*8 D,R2,R3,R4,DIS
      REAL*8 W,XB,QSQ
      LOGICAL ERR
      REAL*8 GET_DIS,GET_BW
      INTEGER IDIS,NDIS
      COMMON/DISFLAG/IDIS,NDIS
      !-------------------------------------------------------------
     
      W   = X1(INDEX)
      QSQ = X2(INDEX)
      XB  = QSQ/(W**2 - M**2 + QSQ) 

      ERR =.FALSE.
      IF(W.LT.(M+MPI)) THEN      ! Check Pi0 threshold
        ERR = .TRUE.
        RETURN
      ENDIF

      DO J=1,NBW
        AR(J) = A(J)
        WR(J) = A(J+4)
        GR(J) = A(J+8)
      ENDDO
      DO J=1,NDIS
        B(J) = A(J+12)
      END DO
      !ALPHA = A(NPARS) ! HERE


      D       = GET_BW(W,QSQ,AR(1),WR(1),GR(1),1)
      R2      = GET_BW(W,QSQ,AR(2),WR(2),GR(2),2)
      R3      = GET_BW(W,QSQ,AR(3),WR(3),GR(3),3)
      R4      = GET_BW(W,QSQ,AR(4),WR(4),GR(4),4)
      DIS     = GET_DIS(IA1A2,XB,QSQ,B(1),B(2),B(3),B(4),B(5))

      FUNCTN  = (D + R2 + R3 + R4) + DIS
      
      RETURN
      END


      REAL*8 FUNCTION GET_BW(W,QSQ,AR,WR,GR,I)
      IMPLICIT NONE

      REAL*8 W,QSQ,AR,WR,GR
      REAL*8 M,MPI
      REAL*8 BW
      REAL*8 QCM,QCR,G,GG
      REAL*8 KCM,KCR
      INTEGER LR,JR,I
      REAL*8 XR

      PARAMETER ( M   = 0.93828 )
      PARAMETER ( MPI = 0.134   )

      ! ---------------------------------------------------
      CALL GET_IX(I,LR,JR,XR)

      QCM   = SQRT(((W**2  + M**2 - MPI**2)/2./W )**2 - M**2)
      KCM   = SQRT(((W**2  + M**2 + QSQ   )/2./W )**2 - M**2)

      QCR   = SQRT(((WR**2 + M**2 - MPI**2)/2./WR)**2 - M**2)
      KCR   = SQRT(((WR**2 + M**2 + QSQ   )/2./WR)**2 - M**2)
      
      G     = GR*(QCM/QCR)**(2*LR+1)*((QCR**2+XR**2)/(QCM**2+XR**2))**LR
      GG    = GR*(KCM/KCR)**(2*JR  )*((KCR**2+XR**2)/(KCM**2+XR**2))**JR

      BW    = AR*(KCR/KCM)**2 * WR**2 * G*GG
      BW    = BW /((WR**2-W**2)**2+(WR*G)**2)

      GET_BW = BW
      RETURN
      END

      SUBROUTINE GET_IX(I,L,J,X)
      IMPLICIT NONE
      ! Return parameters from Stein, Table IX

      INTEGER LR(4),JR(4),I,L,J
      REAL*8 XR(4),X

      DATA LR/ 1, 2, 3, 3/
      DATA JR/ 1, 1, 2, 2/
      DATA XR/ 0.160, 3*0.350/
      ! -------------------------------------

      L = LR(I)
      J = JR(I)
      X = XR(I)

      RETURN
      END


      SUBROUTINE GET_NDIS()
      IMPLICIT NONE
      INTEGER IDIS,NDIS
      COMMON/DISFLAG/IDIS,NDIS

      IF(IDIS.LE.9) THEN
        NDIS    = IDIS            ! Number of DIS parameters
      ELSEIF (IDIS.EQ.21) THEN
        NDIS    = 1
      ELSEIF (IDIS.EQ.22) THEN
        NDIS    = 2
      ELSE
        STOP "PROBBLEM"
      ENDIF
      RETURN
      END


      REAL*8 FUNCTION GET_DIS(I,X,Q2,B1,B2,B3,B4,B5)
      IMPLICIT NONE
      REAL*8 X,B1,B2,B3,B4,B5,Q2
      INTEGER IDIS,I,NDIS
      COMMON/DISFLAG/IDIS,NDIS


      IF (IDIS.EQ.0) THEN                                    
        GET_DIS = 0.
      ELSEIF(IDIS.EQ.1) THEN                                
        GET_DIS = ( B1                            )
      ELSEIF(IDIS.EQ.2) THEN                               
        GET_DIS = ( B1 + B2*X                     ) 
      ELSEIF(IDIS.EQ.3) THEN                              
        GET_DIS = ( B1 + B2*X + B3*X**2           )
      ELSEIF(IDIS.EQ.4) THEN                             
        GET_DIS = ( B1 + B2*X + B3*X**2 + B4*X**3 )
      ELSEIF(IDIS.EQ.5) THEN                            
        GET_DIS = ( B1 + B2*X + B3*X**2 + B4*X**3 ) * X**B5

      
      ELSEIF(IDIS.EQ.21) THEN
        GET_DIS =    X**B1
      ELSEIF(IDIS.EQ.22) THEN
        GET_DIS = B1*X**B2
      
      ELSE
        STOP "PROGLEM"
      ENDIF
      
      IF(I.EQ.2) THEN        ! ONLY FOR A2 = 1/Q DEPENDENCE
        IF(Q2.NE.0.0) THEN
          GET_DIS = GET_DIS/SQRT(Q2)
        ELSE
          GET_DIS = 0.0
        ENDIF
      ENDIF

      RETURN
      END


!======================================================================
      SUBROUTINE FDERIV(X1,X2,I,A,DELTAA,DERIV,NTERMS,IA1A2)
      IMPLICIT NONE
      REAL*8 X1(1),X2(1),A(1),DERIV(1)
      REAL*8 YPLUS,YMINUS,AJ,DELTAA,FUNCTN
      INTEGER I,J,NTERMS,IA1A2




   11 DO 18 J=1,NTERMS
        AJ       = A(J)
        A(J)     = AJ*(1.+DELTAA)
        YPLUS    = FUNCTN(X1,X2,I,A,NTERMS,IA1A2)
        A(J)     = AJ*(1.-DELTAA)
        YMINUS   = FUNCTN(X1,X2,I,A,NTERMS,IA1A2)
        DERIV(J) = (YPLUS-YMINUS)/(2.*DELTAA*AJ)
   18 A(J)     = AJ
      RETURN
      END
    
!======================================================================
      SUBROUTINE FIT(X1,X2,Y,SIGMAY,NPTS,NTERMS,IMODE,A,SIGMAA,YFIT,CHISQR,ERR_MATRIX,IA1A2,DWLIM)

      IMPLICIT NONE
      INCLUDE 'declare.h'
      INTEGER NPTS,NTERMS,IMODE

      REAL*8 X1(NPTS),X2(NPTS),Y(NPTS),SIGMAY(NPTS)
      REAL*8  A(NTERMS),SIGMAA(NTERMS)
      REAL*8  YFIT(NPTS),CHISQR,ERR_MATRIX(MAX,MAX)! ,ERR_MATRIX(NTERMS,NTERMS)

      INTEGER ITER,NTRY,IA1A2,JJ,II
      REAL*8 FLAMDA,CHILST,DWLIM
      LOGICAL ITERATE
      !------------------------------------------------------------------------


      ITER    = 50
      NTRY    = 1
      CHILST  = 1.
      FLAMDA  = 0.001
      ITERATE = .TRUE.
      DWLIM   = 0.15d0 

      DO WHILE (ITERATE)
        
        CALL CURFIT(X1,X2,Y,SIGMAY,NPTS,NTERMS,IMODE,A,SIGMAA,FLAMDA,
     &     YFIT,CHISQR,ERR_MATRIX,IA1A2)
       

        IF ( (NTRY .LT. ITER)  .AND.  (CHISQR .GT. 0.0)  .AND.
     &        ABS((CHISQR - CHILST)/CHILST) .GT. 1.D-5     )THEN
          CHILST  = CHISQR
          NTRY    = NTRY + 1


          IF(IA1A2.eq.2) THEN
            IF(A(9).LE.DWLIM) THEN      ! Test of Delta's width for A2 fit
               NTRY    = NTRY - 1
               ITERATE = .FALSE.
            ENDIF
          ENDIF
          
        ELSE
          ITERATE = .FALSE.
        ENDIF

      ENDDO


      RETURN
      END

!======================================================================
C   SUBROUTINE CURFIT
C
C   PAGE 237 DATA REDUCTION AND ERROR ANALYSIS FOR THE PHYSICAL
C   SCIENCES, PHILIP R. BEVINGTON, MCGRAW-HILL, 1969.
C
C   PURPOSE
C     MAKE A LEAST-SQUARES FIT TO A NON-LINEAR FUNCTION
C     WITH A LINEARIZATION OF THE FITTING FUNCTION.
C
C   USAGE
C     CALL CURFIT(NUMFIT,X,Y,SIGMAY,NPTS,NTERMS,MODE,A,DELTAA,
C                 SIGMAA,FLAMDA,YFIT,CHISQR,ERR_MATRIX)
C
C   DESCRIPTION OF PARAMETERS
C     X      - ARRAY OF DATA POINTS FOR INDEPENDENT VARIABLE
C     Y      - ARRAY OF DATA POINTS FOR DEPENDENT VARIABLE
C     SIGMAY - ARRAY OF STANDARD DEVIATIONS FOR Y DATA POINTS
C     NPTS   - NUMBER OF PAIRS OF DATA POINTS
C     NTERMS - NUMBER OF PARAMETERS
C     IMODE   - DETERMINES METHOD OF WEIGHTING LEASE SQUARES FIT
C             +1 (INSTRUMENTAL) WEIGHT(I) = 1./SIGMAY(I)**2
C              0 (NO WEIGHTING) WEIGHT(I) = 1.
C             -1 (STATISTICAL)  WEIGHT(I) = 1./Y(I)
C     A      - ARRAY OF PARAMETERS
C     DELTAA - ARRAY OF INCREMENTS FOR PARAMETERS A
C     SIGMAA - ARRAY OF STANDARD DEVIATIONS FOR PARAMETERS A
C     FLAMDA - PROPORTION OF GRADIENT SEARCH INCLUDED
C     YFIT   - ARRAY OF CALCULATED VALUES OF Y
C     CHISQR - REDUCED CHI SQUARE FOR FIT
!     ERR_MATRIX = RETURNED TO CALLING ROUTINE
!     ADDED JAN 1987 DBD
C
C     SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C     FUNCTN(NUMFIT,X,I,A,NTERMS)
C       EVALUATES THE FITTING FUNCTION AT X(I)
C     FDERIV(NUMFIT,X,I,A,DERIV,NTERMS)
C       EVALUATES THE DERIVATIVE OF THE FITTING FUNCTION AT X(I)
C       WITH RESPECT TO EACH PARAMETER
C     MATINV(ARRAY,NTERMS,DET)
C       INVERTS A SYMMETRIC TWO DIMENSIONAL MATRIX OF DEGREE NTERMS
C       AND CALCULATES ITS DETERMINANT
C
C     SET FLAMDA = 0.001 AT BEGINNING OF SEARCH

      SUBROUTINE CURFIT(X1,X2,Y,SIGMAY,NPTS,NTERMS,IMODE,A,
     +  SIGMAA,FLAMDA,YFIT,CHISQR,ERR_MATRIX,IA1A2)

      IMPLICIT NONE


      INTEGER NPTS,NTERMS,IMODE
      INTEGER IA1A2

      REAL*8 X1(NPTS),X2(NPTS),Y(NPTS),SIGMAY(NPTS)
      REAL*8 A(NTERMS),SIGMAA(NTERMS),FLAMDA
      REAL*8 YFIT(NPTS),CHISQR,ERR_MATRIX(NTERMS,NTERMS)

      INTEGER NFREE,I,J,K

      REAL*8 ARRAY(NTERMS,NTERMS),DET,DELTAA,WEIGHT(NPTS)
      REAL*8 DERIV(NTERMS),ALPHA(NTERMS,NTERMS),BETA(NTERMS),B(NTERMS)
      REAL*8 CHISQ1
      REAL*8 FUNCTN
      REAL*8 WT

      SAVE 

      PARAMETER (DELTAA=5.D-3)
      
      !------------------------------------------------------------------------
 11   NFREE = NPTS - NTERMS

      IF(NFREE) 13,13,20        ! MODIFIED 8/26/88

 13   CHISQR = 0.0
      
      GO TO 110

!-----EVALUATE WEIGHTS
      
 20   DO 30 I=1,NPTS
 21     IF(IMODE) 22,27,29
 22     IF(Y(I)) 25,27,23
 23     WEIGHT(I) = 1./Y(I)
        GO TO 30
 25     WEIGHT(I) = 1./(-Y(I))
        GO TO 30
 27     WEIGHT(I) = 1.
        GO TO 30
 29     WEIGHT(I) = 1./(SIGMAY(I)**2 )
         WT = WEIGHT(I)
 30   CONTINUE


!-----EVALUATE ALPHA AND BETA MATRICES
31    DO 34 J=1,NTERMS
        BETA(J) = 0.0
        DO 34 K=1,J
34    ALPHA(J,K) = 0.0

41    DO 50 I=1,NPTS
        CALL FDERIV(X1,X2,I,A,DELTAA,DERIV,NTERMS,IA1A2)
        DO 46 J=1,NTERMS
          BETA(J) = BETA(J)+
     +              WEIGHT(I)*(Y(I)-FUNCTN(X1,X2,I,A,NTERMS,IA1A2))*DERIV(J)
          DO 46 K=1,J
            ALPHA(J,K) =ALPHA(J,K)+WEIGHT(I)*DERIV(J)*DERIV(K)
46      CONTINUE
50    CONTINUE

51    DO 53 J=1,NTERMS
        DO 53 K=1,J
          ALPHA(K,J) = ALPHA(J,K)
53    CONTINUE




!-----EVALUATE CHISQR AT STARTING POINT
61    CONTINUE
      CHISQ1 = 0.0
      DO 62 I=1,NPTS
        YFIT(I)  = FUNCTN(X1,X2,I,A,NTERMS,IA1A2)
        !CHISQR   = CHISQR + WEIGHT(I)*(Y(I)-YFIT(I))**2
        CHISQ1   = CHISQ1 + WEIGHT(I)*(Y(I)-YFIT(I))**2          ! KJS
62    CONTINUE
      
      IF(NFREE .NE. 0) THEN 
        !CHISQ1 = CHISQR/NFREE  ! MODIFIED 3/8/88
        CHISQ1 = CHISQ1/NFREE  ! KJS
      ENDIF
      

!-----INVERT MODIFIED CURVATURE MATRIX TO FIND NEW PARAMETERS
71    DO 74 J=1,NTERMS
        DO 73 K=1,NTERMS
73        ARRAY(J,K) = ALPHA(J,K)/SQRT(ALPHA(J,J)*ALPHA(K,K))
          ARRAY(J,J) = 1. + FLAMDA
74    CONTINUE

80    CALL MATINV(ARRAY,NTERMS,DET)

81    DO 84 J=1,NTERMS
        B(J)   = A(J)
        DO 84 K=1,NTERMS
        B(J)   = B(J) + BETA(K)*ARRAY(J,K)/SQRT(ALPHA(J,J)*ALPHA(K,K))
84    CONTINUE


91    CONTINUE
      CHISQR = 0.0
      DO 92 I=1,NPTS
        YFIT(I) = FUNCTN(X1,X2,I,B,NTERMS,IA1A2)
        CHISQR  = CHISQR + WEIGHT(I)*(Y(I)-YFIT(I))**2
92    CONTINUE          


      IF (NFREE .NE. 0) THEN
        CHISQR = CHISQR/NFREE  ! MODIFIED 3/8/88
      ENDIF

      ! Compare new CHISQ to the starting point

      !WRITE(61,'(I5,50F7.3)') IA1A2,A(9)
      
      IF(CHISQ1-CHISQR) 95, 101, 101


95    CONTINUE
      FLAMDA = 10.0*FLAMDA
      IF(FLAMDA.GT.1.0D15) THEN
        GOTO 101
      ENDIF
      GO TO 71
      
!-----EVALUATE PARAMETERS AND UNCERTAINTIES
 101  DO 103 J=1,NTERMS
        A(J)      = B(J)
        SIGMAA(J) = SQRT(ABS(ARRAY(J,J)/ALPHA(J,J)))
 103  CONTINUE
      

!-----FILL UP ERROR MATRIX
      DO I = 1, NTERMS
        DO J = 1, NTERMS
          ERR_MATRIX(I,J) = ARRAY(I,J)/SQRT(ALPHA(I,I)*ALPHA(J,J))
CKJS      ERR_MATRIX(I,J) = SQRT(ABS(ERR_MATRIX(I,J)))
        ENDDO
      ENDDO

        
      FLAMDA = FLAMDA/10.0
 110  CONTINUE

      
      RETURN
      END

!======================================================================
      SUBROUTINE MATINV(ARRAY,NORDER,DET)
      IMPLICIT NONE
CKJS  IMPLICIT REAL*8(A-H,O-Z)
      INTEGER NORDER
      REAL*8 ARRAY(NORDER,NORDER),DET

C   PAGE 302 DATA REDUCTION AND ERROR ANALYSIS FOR
C   THE PHYSICAL SCIENCES, PHILIP R. BEVINGTON, MCGRAW HILL 1969
C
C   PURPOSE:
C     INVERT A SYMMETRIC MATRIX AND CALCULATE ITS DETERMINANT
C
C   USAGE:
C     CALL MATINV(ARRAY,NORDER,DET)
C
C   DESCRIPTION OF PARAMETERS
C     ARRAY  - INPUT MATRIX WHICH IS REPLACED BY ITS INVERSE
C     NORDER - DEGREE OF MATRIX (ORDER OF DETERMINANT)
C     DET    - DETERMINANT OF INPUT MATRIX
C
C   SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C     NONE
C
      INTEGER I,J,K,L
      REAL*8 AMAX,TEMP,IK(NORDER),JK(NORDER)

 10   DET = 1.0
 11   DO 100 K=1,NORDER

!-------FIND LARGEST ELEMENT ARRAY(I,J) IN REST OF MATRIX
        AMAX = 0.0
 21     DO 30 I=K,NORDER
          DO 30 J=K,NORDER
 23         IF (ABS(AMAX) - ABS(ARRAY(I,J))) 24,24,30
 24         AMAX = ARRAY(I,J)
            IK(K) = I
            JK(K) = J
 30     CONTINUE

!-------INTERCHANGE ROWS AND COLUMNS TO PUT AMAX IN ARRAY(K,K)
 31     IF(AMAX) 41,32,41
 32     DET = 0.0
        GO TO 140
 41     I = IK(K)
        IF(I-K) 21,51,43
 43     DO 50 J=1,NORDER
          TEMP = ARRAY(K,J)
          ARRAY(K,J) = ARRAY(I,J)
 50       ARRAY(I,J) = -TEMP
 51     J = JK(K)
        IF(J-K) 21,61,53
 53     DO 60 I=1,NORDER
          TEMP = ARRAY(I,K)
          ARRAY(I,K) = ARRAY(I,J)
 60       ARRAY(I,J) = - TEMP

!-------ACCUMULATE ELEMENTS OF INVERSE MATRIX
 61     DO 70 I=1,NORDER
          IF(I-K) 63,70,63
 63       ARRAY(I,K) = -ARRAY(I,K)/AMAX
 70     CONTINUE
 71     DO 80 I=1,NORDER
          DO 80 J=1,NORDER
            IF(I-K) 74,80,74
 74         IF(J-K) 75,80,75
 75         ARRAY(I,J) = ARRAY(I,J) + ARRAY(I,K)*ARRAY(K,J)
 80     CONTINUE
 81     DO 90 J=1,NORDER
          IF(J-K) 83,90,83
 83       ARRAY(K,J) = ARRAY(K,J)/AMAX
 90     CONTINUE
        ARRAY(K,K) = 1./AMAX
 100    DET = DET * AMAX

!-------RESTORE ORDERING OF MATRIX
 101    DO 130 L=1,NORDER
          K = NORDER - L + 1
          J=IK(K)
          IF(J-K) 111,111,105
 105      DO 110 I=1,NORDER
            TEMP = ARRAY(I,K)
            ARRAY(I,K) = -ARRAY(I,J)
 110        ARRAY(I,J) = TEMP
 111      I=JK(K)
          IF(I-K) 130,130,113
 113      DO 120 J=1,NORDER
            TEMP = ARRAY(K,J)
            ARRAY(K,J) = -ARRAY(I,J)
 120        ARRAY(I,J) = TEMP
 130    CONTINUE
 140    RETURN
        END





      SUBROUTINE GET_DATA(NP,XW_V,Q2_V, A1,EA1, A2,EA2)
      IMPLICIT NONE
      INCLUDE 'declare.h'
      CHARACTER LINE*900

      INTEGER I,J,NP
      REAL*8    XW_V(NX),Q2_V(NX),A1(NX),EA1(NX),A2(NX),EA2(NX)
      REAL*8    XNU,XW,Q2,XBJ,F1TMP,F2TMP,RTMP,
     &
     &        APAR,     EPAR,  APER,    EPER,
     &         DS1,      ES1,   DS2,     ES2,
     &          G1,      EG1,    G2,     EG2,
     &          A1_IN, EA1_IN, A2_IN,    EA2_IN,
     &         D2I,     ED2I,
     &         STT,      ETT,   SLT,     ELT,
     &            FEL1,         FEL2,
     &          ARCEL1,       ARCEL2,
     &         ARCTOT1,      ARCTOT2,
     &        PUREADD1,     PUREADD2,
     &        ETA_V,  BIGD_V,
     &      APAR_D1, APAR_D2,
     &
     &         G2WW,    D2IB,  ED2IB,  GT, EGT,
     &         GDHR,   EGDHR,
     &
     &        W_MID,   W_KIN, Q2_KIN,   X_KIN,  EPS_KIN,
     &      HSCA1PA, HSCA1PE, HSCA2PE,  HSNA1,    HSNA2,
     &         SPAR,    SPER,
     &      XS1_MOD, XS2_MOD,
     & TAIL_UNP_PAR, TAIL_UNP_PER,
     & TAIL_POL_PAR, TAIL_POL_PER


      ! ---------------------------------------------------------------------



      J=0
1000  CONTINUE
        READ(1,'(A)',END=1001) LINE
        
        IF(LINE(1:1).EQ."#") THEN
          CONTINUE
        ELSE
          READ(LINE,*,ERR=1001)
     &      I,XNU,XW,Q2,XBJ,F1TMP,F2TMP,RTMP,            !  1-8
     &      APAR,     EPAR,  APER,    EPER,              !  9-12
     &      DS1,       ES1,   DS2,     ES2,              ! 13-16
     &      G1,       EG1,    G2,     EG2,               ! 17-20
     &      A1_IN,    EA1_IN, A2_IN,  EA2_IN,            ! 21-24
     &      D2I,      ED2I,                              ! 25-26
     &      STT,       ETT,   SLT,     ELT,              ! 27-30
     &      FEL1,      FEL2,                             ! 31-32
     &      ARCEL1,    ARCEL2,                           ! 33-34
     &      ARCTOT1,   ARCTOT2,                          ! 35-36
     &      PUREADD1,  PUREADD2,                         ! 37-38
     &      ETA_V, BIGD_V,                               ! 39-40
     &      APAR_D1,    APAR_D2,                         ! 41-42
     &      G2WW,  D2IB,  ED2IB,  GT, EGT,               ! 43-47
     &      GDHR, EGDHR,                                 ! 48-49
     &      W_MID,   W_KIN, Q2_KIN,  X_KIN,EPS_KIN,      ! 50-54
     &      HSCA1PA, HSCA1PE, HSCA2PE, HSNA1,  HSNA2,    ! 55-59
     &      SPAR,    SPER,                               ! 60-61
     &      XS1_MOD, XS2_MOD,                            ! 62-63
     &      TAIL_UNP_PAR, TAIL_UNP_PER,                  ! 64-65
     &      TAIL_POL_PAR, TAIL_POL_PER                   ! 66-67 
          
            IF(I.EQ.2) THEN
              J       = J+1
              XW_V(J) = XW/1000.
              Q2_V(J) = Q2
              A1(J)   = A1_IN
              EA1(J)  = EA1_IN
              A2(J)   = A2_IN
              EA2(J)  = EA2_IN 
            ENDIF
        ENDIF



      GOTO 1000

1001  CONTINUE
      NP=J
      RETURN
      END



      SUBROUTINE CONVERT(I,CHAR)
      IMPLICIT NONE
      ! 
      ! Stupid subroutine to convert integer to character.
      ! 
      INTEGER I
      CHARACTER CHAR*2

      IF(I.EQ.-1) THEN
        CHAR='-1'
      ELSEIF(I.EQ.0) THEN
        CHAR='00'
      ELSEIF(I.EQ.1) THEN
        CHAR='01'
      ELSEIF(I.EQ.2) THEN
        CHAR='02'
      ELSEIF(I.EQ.3) THEN
        CHAR='03'
      ELSEIF(I.EQ.4) THEN
        CHAR='04'
      ELSEIF(I.EQ.5) THEN
        CHAR='05'
      ELSEIF(I.EQ.6) THEN
        CHAR='06'
      ELSEIF(I.EQ.7) THEN
        CHAR='07'
      ELSEIF(I.EQ.8) THEN
        CHAR='08'
      ELSEIF(I.EQ.9) THEN
        CHAR='09'
      ELSEIF(I.EQ.10) THEN
        CHAR='10'
      ELSEIF(I.EQ.11) THEN
        CHAR='11'
      ELSEIF(I.EQ.12) THEN
        CHAR='12'
      ELSEIF(I.EQ.13) THEN
        CHAR='13'
      ELSEIF(I.EQ.14) THEN
        CHAR='14'
      ELSEIF(I.EQ.15) THEN
        CHAR='15'
      ELSEIF(I.EQ.16) THEN
        CHAR='16'
      ELSEIF(I.EQ.17) THEN
        CHAR='17'
      ELSEIF(I.EQ.18) THEN
        CHAR='18'
      ELSEIF(I.EQ.19) THEN
        CHAR='19'
      ELSEIF(I.EQ.20) THEN
        CHAR='20'
      ELSEIF(I.EQ.21) THEN
        CHAR='21'
      ELSEIF(I.EQ.22) THEN
        CHAR='22'
      ELSEIF(I.EQ.23) THEN
        CHAR='23'
      ELSEIF(I.EQ.24) THEN
        CHAR='24'
      ELSEIF(I.EQ.25) THEN
        CHAR='25'
      ELSEIF(I.EQ.26) THEN
        CHAR='26'
      ELSEIF(I.EQ.27) THEN
        CHAR='27'
      ELSEIF(I.EQ.28) THEN
        CHAR='28'
      ELSEIF(I.EQ.29) THEN
        CHAR='29'
      ELSE
        WRITE(6,*) I
        STOP "PROBLEM CONVERT"
      ENDIF

      RETURN
      END

